import { useState } from 'react';
import { Form } from 'react-bootstrap';

/**
 * Validate password text.
 *
 * @param value The password text.
 * @param reValue The re-inputed password text.
 * @param required The required flag.
 * @returns If there are no problems, an empty string is returned.
 */
const validateText = (value: string, reValue: string, required: boolean) => {
  if ((value.length > 0 || reValue.length > 0) && (value !== reValue)) {
    return 'The specified passwords do not match.';
  }
  if (required && value.length === 0) {
    return 'Password field is required.';
  }

  return '';
}

/**
 * Block of key-value input view.
 */
const PasswordInputBlock = ({ label, confirmLabel, value, setValue, required, readOnly }: {
  label: string,
  confirmLabel: string,
  value: string,
  setValue: (s: string) => void,
  required: boolean,
  readOnly: boolean,
}) => {

  const [reInputValue, setInputValue] = useState('');
  const [errorMessage, setErrorMessage] = useState('');

  const setValueWrapper = (v: string) => {
    setValue(v);

    setErrorMessage(validateText(v, reInputValue, required));
  }

  const setInputValueWrapper = (v: string) => {
    setInputValue(v);

    setErrorMessage(validateText(value, v, required));
  }

  return <>
    <Form.Group className="form-item js-form-item">
      <Form.Label className={'control-label' + (required ? ' form-required' : '')}>{label}</Form.Label>
      <input className="form-text form-control" type="password" value={value} onChange={(e) => {
        setValueWrapper(e.currentTarget.value);
      }} readOnly={readOnly} />
    </Form.Group>
    <Form.Group className="form-item js-form-item">
      <Form.Label className={'control-label' + (required ? ' form-required' : '')}>{confirmLabel}</Form.Label>
      <input className="form-text form-control" type="password" value={reInputValue} onChange={(e) => {
        setInputValueWrapper(e.currentTarget.value);
      }} readOnly={readOnly} />
      {
        errorMessage !== ''
          ? <Form.Text className="text-danger fw-bold">
            Error: {errorMessage}
          </Form.Text>
          : <></>
      }
    </Form.Group>
  </>;

}

export default PasswordInputBlock;
