<?php

namespace Drupal\Tests\cloud\Behat\features\bootstrap;

use Drupal\DrupalExtension\Context\RawDrupalContext;
use Drupal\DrupalExtension\MinkAwareTrait;
use Behat\Mink\Element\NodeElement;

/**
 * Defines application features from the specific context.
 */
class FeatureContext extends RawDrupalContext {

  use MinkAwareTrait;

  /**
   * Initializes context.
   *
   * Every scenario gets its own context instance.
   * You can also pass arbitrary arguments to the
   * context constructor through behat.yml.
   */
  public function __construct() {
  }

  /**
   * Fill in the hour/minute based on the elapsed minute from the current time.
   *
   * @var hour_id
   *  The ID of the hour list.
   * @var minute_id
   *  The ID of the minute list.
   * @var elapsed_minute
   *  The elapsed minute.
   *
   * @throws \Exception
   *   If no hour or minute cannot be selected.
   */
  private function selectTimeByElapsedTime(string $hour_id, string $minute_id, string $elapsed_minute): void {
    $element = $this->getSession()->getPage();
    $hour_select = $element->findById($hour_id);
    $minute_select = $element->findById($minute_id);

    if (empty($hour_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $hour_id, $this->getSession()->getCurrentUrl()));
    }
    if (empty($minute_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $minute_id, $this->getSession()->getCurrentUrl()));
    }

    $time = new \DateTime('now', new \DateTimeZone(date_default_timezone_get()));
    $time->add(\DateInterval::createFromDateString("$elapsed_minute minute"));
    $hour = $time->format('H');
    $minute = $time->format('i');

    $hour_select->selectOption($hour, FALSE);
    $minute_select->selectOption($minute, FALSE);
  }

  /**
   * Wait until the specified schedule time.
   *
   * @var hour_id
   *  The ID of the hour list.
   * @var minute_id
   *  The ID of the minute list.
   *
   * @throws \Exception
   *   If the selector for hour_id or minute_id is not found.
   */
  private function waitUntilScheduleTime(string $hour_id, string $minute_id): void {
    $resource_startup_wait_minute = 1;
    $element = $this->getSession()->getPage();
    $hour_select = $element->findById($hour_id);
    $minute_select = $element->findById($minute_id);

    if (empty($hour_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $hour_id, $this->getSession()->getCurrentUrl()));
    }
    if (empty($minute_select)) {
      throw new \Exception(sprintf('The select with "%s" was not found on the page %s', $minute_id, $this->getSession()->getCurrentUrl()));
    }

    $hour = $hour_select->getValue();
    $minute = $minute_select->getValue();

    if (!is_numeric($hour) || !(0 <= $hour && $hour <= 23)) {
      throw new \Exception(sprintf('Failed to get the valid value of %s from the hour select.', $hour_id));
    }
    if (!is_numeric($minute) || !(0 <= $minute && $minute <= 59)) {
      throw new \Exception(sprintf('Failed to get the valid value of %s from the minute select.', $minute_id));
    }

    $now_time = new \DateTime('now', new \DateTimeZone(date_default_timezone_get()));
    $wait_until_time = new \DateTime('', new \DateTimeZone(date_default_timezone_get()));
    $wait_until_time->setTime($hour, $minute);
    $wait_until_time->add(\DateInterval::createFromDateString("$resource_startup_wait_minute Minutes"));
    $wait_time = $wait_until_time->diff($now_time);
    // Converts hours, minutes, and seconds to seconds.
    $wait_second = $wait_time->h * 60 * 60 + $wait_time->i * 60 + $wait_time->s;
    sleep($wait_second);
  }

  /**
   * Select the start time in elapsed minute from the current time.
   *
   * @When I select :elapsed_minutes minutes later now from the start-up time
   */
  public function selectScheduleStartupTimeByElapsedTime($elapsed_minutes) {
    $this->selectTimeByElapsedTime('edit-start-hour', 'edit-start-minute', $elapsed_minutes);
  }

  /**
   * Select the stop time in elapsed minute from the current time.
   *
   * @When I select :elapsed_minutes minutes later now from the stop time
   */
  public function selectScheduleStopTimeByElapsedTime($elapsed_minutes) {
    $this->selectTimeByElapsedTime('edit-stop-hour', 'edit-stop-minute', $elapsed_minutes);
  }

  /**
   * Wait until the stop time for Schedule.
   *
   * @When I wait until start-up time for schedule
   */
  public function waitUntilStartupTimeForSchedule(): void {
    $this->waitUntilScheduleTime('edit-start-hour', 'edit-start-minute');
  }

  /**
   * Wait until the stop time for Schedule.
   *
   * @When I wait until stop time for schedule
   */
  public function waitUntilStopTimeForSchedule(): void {
    $this->waitUntilScheduleTime('edit-stop-hour', 'edit-stop-minute');
  }

  /**
   * Get a row by the rowIndex in the specified region.
   *
   * @param int $rowIndex
   *   The row index.
   * @param string $region
   *   The search region.
   *
   * @return \Behat\Mink\Element\NodeElement
   *   The NodeElement in the row.
   *
   * @throws \Exception
   *   If no rowObj is found.
   */
  private function getRowByIndexInRegion(int $rowIndex, string $region): NodeElement {
    $regionObj = $this->getRegion($region);
    $tableRowSelector = sprintf('.row-%d', (int) $rowIndex);
    $rowObj = $regionObj->find('css', $tableRowSelector);
    if (empty($rowObj)) {
      throw new \Exception(sprintf('No field "%s" found in the region %s of %s.', $tableRowSelector, $region, $this->getSession()->getCurrentUrl()));
    }
    return $rowObj;
  }

  /**
   * Fills in a form field with id|name|title|alt|value in the specified row.
   *
   * The row is specified by the index in the region.
   *
   * @When I fill in :field with :value in the row :rowIndex in the :region (region)
   */
  public function fillFieldInTableRowInRegion($value, $field, $rowIndex, $region): void {
    $this->getRowByIndexInRegion($rowIndex, $region)->fillField($field, $value);
  }

  /**
   * Selects an option in select field with specified id|name|title|alt|value.
   *
   * The select field is also specified by the row index in the region.
   *
   * @When I select :option from :field in the row :rowIndex in the :region (region)
   */
  public function selectOptionInTableRowInRegion($option, $field, $rowIndex, $region): void {
    $this->getRowByIndexInRegion($rowIndex, $region)->selectFieldOption($field, $option);
  }

}
